#!/usr/bin/perl

# make index title 

if ($ARGV[1] eq "") { usage(); };

$indname = $ARGV[0];
$ifname =  $ARGV[1];

readindex($indname);
#  printf "##lno = %d##\n", $lno;

if (!open(IFILE, $ifname)) {
        print "Cannot open source file '",$ifname,"'.\n";
        exit;
};

$ofile="";

$smode=-1; # initialize section mode
$bmode=0; # body mode | 0:first line, 1:normal body, 2:<PRE>, 3:<TABLE> 4:<P>

$buf="";
$fmsg="";

$lno=0; # section counter. (for index database array)

while (defined($line = <IFILE>)) {
  chomp $line;

#printf "#%d#%d#%s#\n",$smode,$bmode,$line;

  if($line =~ /^Sec(\d): (\w+) +/ ) {  # Section line

    $lno++; # index database line number
    $lvl=$1; # Section level
    $lnk=$2; # link target
    # $msg=$'; # section title

    if($lnk eq "#") { # Comment out whole section. Skip to the next section.
      $smode = 0;
    } else {
      $smode=1; # open file mode.
      if($lnk eq "_") { # section don't request page division.
        # nothing to do.
      } elsif($lstr[$lno] eq "_") { # new section with page divede
        purger($buf);
        trailer($lno-1, $fmsg); # end of file.
        $fmsg=""; $fcnt=0; # init footer 
        header($lno);
      } else { # new section witout page division 
	# place a mark 
        printf OFILE "<a name=\"%s\"></a>\n", $lstr[$lno];
      };

      # make Section title.
      if($lvl eq 1) {
        printf OFILE "<p class=\"title1\"><font size=\"+2\">%s %s</font>",$sstr[$lno],$msg[$lno];
      } elsif($lvl eq 2) {
        printf OFILE "<p class=\"title2\"><font size=\"+1\">%s %s</font>",$sstr[$lno],$msg[$lno];
      } elsif($lvl eq 3) {
        printf OFILE "<p class=\"title3\"><font><b>%s</b></font>",$msg[$lno];
      } elsif($lvl eq 4) {
        printf OFILE "<p class=\"title4\"><font><b>%s</b></font>",$msg[$lno];
      } else {
        printf OFILE "<p class=\"title5\"><font><u>%s</u></font>",$msg[$lno];
      };
      printf OFILE "</p>\n\n";

      $buf=""; # init body buffer.
    };
    

  } else { # body data.
    if($smode < 1) { # init or skip mode.
      # nothing to do.
    } else { # body handling.
      if($bmode eq 0 or $bmode eq 1) { # normal paragraphs.
        if($line =~ /^#/ ) {  # comment. just skip it.
	  # nothing to do
        } elsif($line =~ /<PRE>/i ) {  # Start <PRE> section.
          purger($buf); $buf=""; 
          printf OFILE "%s\n",$line;
          $bmode=2;
        } elsif($line =~ /<TABLE /i ) {  # Start <TABLE> section.
          purger($buf); $buf=""; 
          printf OFILE "%s\n",$line;
          $bmode=3;
        } elsif($line =~ /<P /i ) {  # Start <P> section.
          purger($buf); $buf=""; 
          printf OFILE "%s\n",$line;
          $bmode=4;
        } elsif($line =~ /^ *$/ ) {  # Blank line.
          if($bmode eq 0) { # before any effective body line. Skip it.
            # nothing to do.
          } else { # bmode should be 1, after body lines.
            purger($buf); $buf=""; 
            $bmode=0;
          };
        } else { # normal body message.          
          $line2=bodyproc($line); 

          # tag only line. "<img src....>" will put as is.
          $la = $line2; $la =~ s/<[^>]*>//g; $la =~ s/[ 	]*//g;
          if($la eq "") { 
            purger($buf); $buf=""; 
            $bmode=0;
            printf OFILE "%s\n",$line2;

          } else {
            if($bmode eq 0) { # first line. 
              $buf=sprintf("%s%s",$buf,$line2);
            } else {
              $buf=sprintf("%s<br>\n%s",$buf,$line2);
            };
            $bmode=1;

          };
        };

      } elsif($bmode eq 2) { # <PRE> mode, just put as is.
        printf OFILE "%s\n",$line;
        if($line =~ /^ *<\/PRE> *$/i ) {  # End of <PRE> mode.
          $bmode=0;
        };

      } elsif($bmode eq 3) { # <TABLE> mode, just put as is.
        printf OFILE "%s\n",$line;
        if($line =~ /^ *<\/TABLE> *$/i ) {  # End of <TABLE> mode.
          $bmode=0;
        };

      } elsif($bmode eq 4) { # <P> mode, just put as is.
        printf OFILE "%s\n",$line;
        if($line =~ /^ *<\/P> *$/i ) {  # End of <P> mode.
          $bmode=0;
        };

      } else { # unknown mode?
        printf "unknown node\n";
      };

    };
  };
}

purger($buf);
trailer($lno, $fmsg); # end of file.

close(IFILE);

exit;

# read index database.
sub readindex {
  $indname=$_[0];

  if (!open(INDFILE, $indname)) {
        print "Cannot open index database file '%s'.\n", $indname;
        exit;
  };

  $lno=0;
  while (defined($line = <INDFILE>)) {
    chomp $line;
    $lno++;

    if( $line =~ /^(\d) +([\w\.]+) +(\w+) +(\w+) +(\w+) +/ ) {
      $lvl[$lno]=$1;   # Section level  
      $sstr[$lno]=$2;  # Section string
      $hstr[$lno]=$3;  # html name
      $lstr[$lno]=$4;  # link name
      $pmark[$lno]=$5; # page break mark
      $msg[$lno]=$';   # section title

      if( $4 eq "_" ) { # keep unique label to $lbl hashed-array.
        $lbl{$3}=$lno;
      } else {
        $lbl{$4}=$lno;
      };
    } else {
      printf "##\n";
      printf "##\n";
      printf "Something Wrong in index database file. (%s)\n", $line;
      printf "##\n";
      printf "##\n";
    };
  };
  close(INDFILE);

  # while (($key,$value) = each(%lbl)) {
  #    print "key=$key, value=$value\n";
  # }

}


# purge out body buffer.
sub purger {
  $b1=$_[0];

  if($b1 eq "") {
    # nothing to do.
  } else {
    printf OFILE "<p>\n%s\n</p>\n\n",$b1;
  };

  return;
}


# message body conversion.
sub bodyproc {
  $l=$_[0];
  $ll="";

  # << Reference >> processing.
  do {
    if($l =~ /<<([_a-zA-Z0-9-]+):([^>]*)>>/ ) {  # find Reference.
      $i=$lbl{$1}; # index database line number of this reference.
      if($lstr[$lbl{$1}] eq "_") { 
        $s="_".$hstr[$lbl{$1}].".html";
      } else {
        $s="_".$hstr[$lbl{$1}].".html#".$lstr[$lbl{$1}];
      };
      $ll=sprintf "%s%s<a target=\"body\" href=\"%s\">%s</a>",
                  $ll,$`,$s,$2;
      $l=$';
    } else {
      $ll=sprintf "%s%s",$ll,$l;
      $l="";
    };
  } until ($l eq "");

  $l=$ll;

  # [[ Footnote ]] processing.
  do {
    if($l =~ /\[\[([^\]]*)\]\]/ ) {  # find Footnote.
      $fcnt+=1;
      $fmsg=sprintf("%s<i>[%d]</i> %s<br>\n",$fmsg,$fcnt,$1); 
      $ll=sprintf "%s%s <font size=\"-1\"><sup>[%d]</sup></font> \n",$ll,$`,$fcnt;
      $l=$';
    } else {
      $ll=sprintf "%s%s",$ll,$l;
      $l="";
    };
  } until ($l eq "");

  return($ll);
}

sub usage {
  print "usage: mkdocs_b.pl index.dat body.txt\n";
  print "  read index.dat and body.txt then break out leaf pages in HTML. \n";
  exit;
}

sub header {
  $lno=$_[0];

  $ofname="_".$hstr[$lno].".html";

  if (!open(OFILE, ">$ofname")) {
        print "Cannot open output file '",$ofname,"'.\n";
        exit;
  };

  if (!open(IIFILE, "mkdocs_bhead.dat")) {
        print "Cannot open header file 'mkdocs_bhead.dat'.\n";
        exit;
  };
  while (defined($line = <IIFILE>)) {
    chomp $line;
    printf OFILE "%s\n",$line;
  };
  close(IIFILE);
  
};

sub trailer {
  $ll=$_[0]; # index number
  $ff=$_[1]; # footer message buffer

  # write footer message
  if($ff ne "") {
    printf OFILE "<p>\n<hr noshade width=\"95\%\" size=\"1\"><ul>\n";
    printf OFILE "<font size=\"-1\">\n";
    printf OFILE "%s",$ff;  # output fotter message.
    printf OFILE "</font></ul></p>\n";
  };

  # link to content frame.
  printf OFILE "<div align=\"right\">\n";
  printf OFILE "<a target=\"contents\" href=\"_contents.html#%s\"><img border=\"0\" src=\"aback.gif\"></a>\n",$hstr[$ll];
  printf OFILE "<img src=\"space.gif\">\n";
  
  # link to the recent page (this is trick!)
  for($i=$ll; $i>0; $i--)  {
    if($pmark[$i] eq "p") { # pagetop found. 
      if( $i ne 1 ) { # Place a link to 1 page before.
        printf OFILE "<a target=\"body\" href=\"_%s.html\"><img border=\"0\" src=\"aup.gif\"></a>\n",$hstr[$i-1];
          $i=0;
      };
    };
  };

  # link to the next page.
  if($hstr[$ll+1] ne "") { 
    printf OFILE "<a target=\"body\" href=\"_%s.html\"><img border=\"0\" src=\"adown.gif\"></a>\n",$hstr[$ll+1];
  };
  printf OFILE "</div>\n";

  if (!open(IIFILE, "mkdocs_btail.dat")) {
        print "Cannot open header file 'mkdocs_btail.dat'.\n";
        exit;
  };
  while (defined($line = <IIFILE>)) {
    chomp $line;
    printf OFILE "%s\n",$line;
  };
  close(IIFILE);

  close(OFILE);
}

